// Copyright 2012, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Hannah Bast <bast@informatik.uni-freiburg.de>.

import java.util.ArrayList;

/**
 * Class for intersecting two sorted lists of integers, using the linear-time
 * algorithm. Used for demonstrating the effects of relatively minor code
 * changes on the running time.
 */
public class Intersect {

  /**
   * Intersect two lists given as native int arrays. Version 3.
   */
  public static int[] intersectV3(int[] list1, int[] list2) {
    int[] result = new int[Math.min(list1.length, list2.length)];
    int i = 0;
    int j = 0;
    int k = 0;
    while (i < list1.length && j < list2.length) {
      while (i < list1.length && list1[i] < list2[j]) { i++; }
      while (j < list2.length && list1[i] > list2[j]) { j++; }
      if (list1[i] == list2[j]) {
        result[k] = list1[i];
        i++;
        j++;
        k++;
      }
    }
    return result;
  }

  /**
   * Intersect two lists given as native int arrays. Version 2.
   */
  public static int[] intersectV2(int[] list1, int[] list2) {
    int[] result = new int[Math.min(list1.length, list2.length)];
    int i = 0;
    int j = 0;
    int k = 0;
    while (i < list1.length && j < list2.length) {
      if (list1[i] < list2[j]) {
        i++;
      } else if (list1[i] > list2[j]) {
        j++;
      } else {
        // list1[i] == list2[j].
        result[k] = list1[i];
        i++;
        j++;
        k++;
      }
    }
    return result;
  }

  /**
   * Intersect two lists given as native int arrays. Version 1.
   */
  public static ArrayList<Integer> intersectV1(int[] list1, int[] list2) {
    ArrayList<Integer> result = new ArrayList<Integer>();
    int i = 0;
    int j = 0;
    while (i < list1.length && j < list2.length) {
      if (list1[i] < list2[j]) {
        i++;
      } else if (list1[i] > list2[j]) {
        j++;
      } else {
        // list1[i] == list2[j].
        result.add(list1[i]);
        i++;
        j++;
      }
    }
    return result;
  }

  /**
   * Intersect two lists given as ArrayList objects.
   */
  public static ArrayList<Integer> intersect(
      ArrayList<Integer> list1, ArrayList<Integer> list2) {
    ArrayList<Integer> result = new ArrayList<Integer>();
    int i = 0;
    int j = 0;
    while (i < list1.size() && j < list2.size()) {
      if (list1.get(i) < list2.get(j)) {
        i++;
      } else if (list1.get(i) > list2.get(j)) {
        j++;
      } else {
        // list1[i] == list2[j].
        result.add(list1.get(i));
        i++;
        j++;
      }
    }
    return result;
  }

}
