// Copyright 2012, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Hannah Bast <bast@informatik.uni-freiburg.de>.

import java.util.ArrayList;
import java.text.DecimalFormat;

/**
 * Class for timing our intersection on some example list.
 */
public class IntersectMain {

  /**
   * Main function.
   */
  public static void main(String[] args) {
    // Parse command line arguments.
    if (args.length != 2) {
      System.out.println(
          "Usage: java -jar IntersectMain <#elements list 1> <ratio list 2>");
      System.exit(1);
    }
    int n = Integer.parseInt(args[0]);
    int k = Integer.parseInt(args[1]);
    DecimalFormat df = new DecimalFormat("0.0");
    long startTime;
    long endTime;
    int numRuns = 3;

    // Generate two lists as follows:
    // list1: 0, k, 2k, 3k, ..., (n - 1)k   (n elements)
    // list2: 0, 1, 2, 3, ..., nk - 1     (nk elements)
    System.out.print("Generating two lists of size "
        + df.format(n / 1e6) + "M and " + df.format(n * k / 1e6) + "M ... ");
    startTime = System.currentTimeMillis();
    int[] list1 = new int[n];
    int[] list2 = new int[n * k];
    for (int i = 0; i < n; i++) { list1[i] = i * k; }
    for (int j = 0; j < n * k; j++) { list2[j] = j; }
    // ArrayList<Integer> list1 = new ArrayList<Integer>(n);
    // ArrayList<Integer> list2 = new ArrayList<Integer>(n * k);
    // for (int i = 0; i < n; i++) { list1.add(i * k); }
    // for (int j = 0; j < n * k; j++) { list2.add(j); }
    endTime = System.currentTimeMillis();
    assert list1.length == n;
    assert list2.length == n * k;
    // assert list1.size() == n;
    // assert list2.size() == n * k;
    System.out.println("done in "
        + df.format((endTime - startTime) / 1000.0) + "s");

    // Intersect the two lists (version 1).
    System.out.println();
    for (int run = 1; run <= numRuns; run++) {
      System.out.print("Intersecting the two lists (version 1) ... ");
      startTime = System.currentTimeMillis();
      ArrayList<Integer> result = Intersect.intersectV1(list1, list2);
      endTime = System.currentTimeMillis();
      System.out.println("done in "
          + "\033[1m" + df.format((endTime - startTime) / 1000.0) + "s\033[0m");
      assert result.size() == n;
    }

    // Intersect the two lists (version 2).
    System.out.println();
    for (int run = 1; run <= numRuns; run++) {
      System.out.print("Intersecting the two lists (version 2) ... ");
      startTime = System.currentTimeMillis();
      int[] result = Intersect.intersectV2(list1, list2);
      endTime = System.currentTimeMillis();
      System.out.println("done in "
          + "\033[1m" + df.format((endTime - startTime) / 1000.0) + "s\033[0m");
      assert result.length == n;
    }

    // Intersect the two lists (version 3).
    System.out.println();
    for (int run = 1; run <= numRuns; run++) {
      System.out.print("Intersecting the two lists (version 3) ... ");
      startTime = System.currentTimeMillis();
      int[] result = Intersect.intersectV3(list1, list2);
      endTime = System.currentTimeMillis();
      System.out.println("done in "
          + "\033[1m" + df.format((endTime - startTime) / 1000.0) + "s\033[0m");
      assert result.length == n;
    }
  }
}
